/**
 ** hrp2Hrp2Codels.c
 **
 ** Codels called by execution task hrp2Hrp2
 **
 ** Author:
 ** Date:
 **
 **/

#include <portLib.h>
#include "server/hrp2Header.h"

#include <iostream>

#include "netshring.h"
#include "genom-openhrp-plugin-idl.h"
#include "stateProvider-idl.h"
#include "seqplugin-idl.h"
#include "joints.h"
#include "pom.h"
#include "t3d/t3d.h"

using namespace ::std;
using namespace ::genom::hrp2;


/* --- local data ---------------------------------------------------- */

static CORBA::ORB_var orb;
static GenomOpenHRP_var genom_plugin;
static stateProvider_var openHRPsp;
static seqplugin_var openHRPseq;

/** current robot state, exported by stateProvider plugin. */
static RobotState_var robotState;

/** fifo of pending configs to send to OpenHRP. */
static GenomConfigList configs;

/** XXX temp */
static POSTER_ID	pomPosId;


/* --- hrp2Hrp2Init -------------------------------------------------- */

/** \brief Initialization codel (fIDS, ...)
 *
 * \param[out] report	Execution status
 *
 * \return OK or ERROR
 */

STATUS
hrp2Hrp2Init(int *report)
{
  SDI_F->initialized = 0;

  /* init odometry to 0. for now */
  pomPosId = NULL;
  SDI_F->odometry.kind = POM_ME_ABSOLUTE;
  SDI_F->odometry.unused = 0;
  SDI_F->odometry.confidence = 1.;
  SDI_F->odometry.date1 = 0;
  SDI_F->odometry.date2 = 0;
  memset(&SDI_F->odometry.main, 0, sizeof(SDI_F->odometry.main));
  memset(&SDI_F->odometry.VLocal, 0, sizeof(SDI_F->odometry.VLocal));

  /* Compute static pom information */
  for(int i = 0; i < HRP2_MAXLINK; i++)
    strncpy(SDI_F->pomSE[pomLinkNames[i].value].seName, pomLinkNames[i].name,
	    sizeof(SDI_F->pomSE[pomLinkNames[i].value].seName));

  /* Initialize pending configs */
  SDI_F->time = 0.;
  configs.length(0);

  fprintf(stderr, "task initialized at %gms\n", (1.0/sysClkRateGet())*1000.);
  return OK;
}


/*------------------------------------------------------------------------
 *
 * hrp2Hrp2End  --  Termination codel
 *
 * Description:
 *
 * Returns:    OK or ERROR
 */

STATUS
hrp2Hrp2End()
{
  if (SDI_F->initialized)
    orb->shutdown(1);
  return OK;
}


/* --- hrp2Hrp2Perm -------------------------------------------------- */

/** \brief Codel of permanent activity
 *
 * \param[out] report		OK
 *
 * \returns OK or ERROR
 */

STATUS
hrp2Hrp2Perm(int *report)
{
  T3D waist, chest;

  if (!SDI_F->initialized) return OK;

  if (!pomPosId) posterFind("pomPos", &pomPosId);
  if (pomPosId) {
    POM_POS pos;
    posterRead(pomPosId, 0, &pos, sizeof(pos));
    SDI_F->odometry.date1 = pos.pomTickDate;
  }

  try {
    CORBA::Double percent;

    /* push as many configs as possible */
    if (configs.length() > 0) {
      CORBA::Long n = genom_plugin->push(configs, percent);
      if (n > 0) {
	for(unsigned int i = n; i < configs.length(); i++) {
	  configs[i-n] = configs[i];
	}
	configs.length(configs.length()-n);
	cerr << "sent " << n << " configs" << endl;
      }
    }

    /* get state */
    openHRPsp->getActualState(robotState);
  }

  catch (CORBA::Exception& ex) {
    SDI_F->initialized = 0;
    cerr << "Disconnecting after CORBA::Exception:" << ex._name() << endl;
    return OK;
  }

  /* copy state to poster */
  for(int i = 0; i < GHRP2_MAXDOF; i++)
    SDI_F->config.angles[jointNames[i].genom] = robotState->angle[jointNames[i].openHRP];

  /* compute waist orientation */
  waist = t3dBryanIdentity;
  waist.euler.euler[3] = waist.euler.euler[4] = waist.euler.euler[5] = 0.;
  waist.euler.euler[0] = SDI_F->config.angles[GHRP2_CHEST_JOINT0];
  waist.euler.euler[1] = SDI_F->config.angles[GHRP2_CHEST_JOINT1];
  waist.euler.euler[2] = 0.0;
  t3dConvertTo(T3D_MATRIX, &waist);
  t3dInvert(&waist);

  chest = t3dBryanIdentity;
  chest.euler.euler[3] = chest.euler.euler[4] = chest.euler.euler[5] = 0.;
  chest.euler.euler[0] = robotState->attitude[0][2];
  chest.euler.euler[1] = robotState->attitude[0][1];
  chest.euler.euler[2] = robotState->attitude[0][0];
  t3dConvertTo(T3D_MATRIX, &chest);

  t3dComp(&waist, &chest);
  t3dConvertTo(T3D_EULER, &waist);

  SDI_F->config.waistRpy[2] = waist.euler.euler[0];
  SDI_F->config.waistRpy[1] = waist.euler.euler[1];
  SDI_F->config.waistRpy[0] = waist.euler.euler[2];

  SDI_F->odometry.main.euler.yaw = waist.euler.euler[0];
  SDI_F->odometry.main.euler.pitch = waist.euler.euler[1];
  SDI_F->odometry.main.euler.roll = waist.euler.euler[2];

  /* compute pom SEs */
  hrp2PomGetPanTiltLink(SDI_F->config.angles[GHRP2_HEAD_JOINT0],
			SDI_F->config.angles[GHRP2_HEAD_JOINT1],
			&SDI_F->pomSE[HRP2_HEAD_TO_NECK].seConfig);

  hrp2PomGetPanTiltLink(SDI_F->config.angles[GHRP2_CHEST_JOINT0],
			SDI_F->config.angles[GHRP2_CHEST_JOINT1],
			&SDI_F->pomSE[HRP2_CHEST_TO_WAIST].seConfig);
  return OK;
}


/* --- hrp2InitExec -------------------------------------------------- */

/** \brief codel EXEC of Init
 *
 * \param[out] report		OK
 *				S_hrp2_OPENHRP_NOT_FOUND
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2InitExec(int *report)
{
  CORBA::Object_var obj;
  CosNaming::NamingContext_var root;
  PluginManager_var pm;

  /* connect to CORBA services */

  try {
    int argc = 0;
    orb = CORBA::ORB_init(argc, NULL);

    obj = orb->resolve_initial_references("NameService");
    root = CosNaming::NamingContext::_narrow(obj);
    if( CORBA::is_nil(root) ) {
      cerr << "Failed to narrow the root naming context." << endl;
      *report = S_hrp2_NAMESERVER_NOT_FOUND;
      return ETHER;
    }
  }

  catch (CORBA::NO_RESOURCES&) {
    cerr << "Caught NO_RESOURCES exception. You must configure omniORB "
	 << "with the location" << endl
	 << "of the naming service." << endl;
    *report = S_hrp2_NAMESERVER_NOT_FOUND;
    return ETHER;
  }

  catch(CORBA::Exception& ex) {
    cerr << "Caught CORBA::Exception:" << ex._name() << endl;
    *report = S_hrp2_CORBA_NOT_FOUND;
    return ETHER;
  }

  try {
    CosNaming::Name name;

    // Plugin manager
    name.length(1);
    name[0].id = CORBA::string_dup("motionsys");
    pm = PluginManager::_narrow(root->resolve(name));
    cout << "Found OpenHRP plugin manager" << endl;

    // State Provider
    pm->load("provider");
    pm->create("provider", "sp", "");

    name.length(1);
    name[0].id = CORBA::string_dup("sp");
    openHRPsp = stateProvider::_narrow(root->resolve(name));
    openHRPsp->start();
    cout << "Started OpenHRP state provider" << endl;

    // Genom plugin
    name.length(1);
    name[0].id = CORBA::string_dup("genom-openhrp");
    genom_plugin = GenomOpenHRP::_narrow(root->resolve(name));
    genom_plugin->start();
    cout << "Started genom plugin" << endl;

    // Seq. plugin
    pm->load("seqplay");
    pm->create("seqplay", "seq", "");
    cout << "Loaded OpenHRP seq. plugin" << endl;

    name.length(1);
    name[0].id = CORBA::string_dup("seq");
    obj = root->resolve(name);
    openHRPseq = seqplugin::_narrow(obj);
    openHRPseq->start();
    cout << "Started OpenHRP seq. plugin" << endl;
  }

  catch(omniORB::fatalException& fe) {
    cerr << "Caught omniORB::fatalException:" << endl;
    cerr << " file: " << fe.file() << endl;
    cerr << " line: " << fe.line() << endl;
    cerr << " mesg: " << fe.errmsg() << endl;

    *report = S_hrp2_OPENHRP_NOT_FOUND;
    return ETHER;
  }

  catch(CORBA::Exception& ex) {
    cerr << "Caught CORBA::Exception:" << ex._name() << endl;
    *report = S_hrp2_OPENHRP_NOT_FOUND;
    return ETHER;
  }

  SDI_F->initialized = 1;
  cout << "Module initialized" << endl;
  return ETHER;
}


/* --- SetJointAngle ------------------------------------------------- */

/** \brief codel START of SetJointAngle
 *
 * \param[in] setJointAngle	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetJointAngleStart(GHrp2SetJointAngle *setJointAngle, int *report)
{
  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

#if 0
  openHRPseq->setJointAngle(jointNames[setJointAngle->joint].jointname,
			    setJointAngle->angle, setJointAngle->delay);
  return EXEC;
#endif

  GenomConfigList cl;
  CORBA::Double p;
  CORBA::Long n;

  cl.length(1);
  cl[0].angles.length(40);
  cl[0].zmp.length(3);
  cl[0].waistRpy.length(3);

  for(unsigned int i = 0; i< 40; i++)
    cl[0].angles[i] = 0.;
  cl[0].zmp[0] = cl[0].zmp[1] = cl[0].zmp[2] = 0.;
  cl[0].waistRpy[0] = cl[0].waistRpy[1] = cl[0].waistRpy[2] = 0.;
  cl[0].time = 20.0;

  n = genom_plugin->push(cl, p);
  printf("plugin: %ld %g\n", (long)n, (double)p);

  return ETHER;
}

/** \brief codel EXEC of SetJointAngle
 *
 * \param[in] setJointAngle	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetJointAngleExec(GHrp2SetJointAngle *setJointAngle, int *report)
{
  if (openHRPseq->isEmpty()) return ETHER;
  return EXEC;
}


/* --- GetLinkPos ---------------------------------------------------- */

/** \brief codel EXEC of GetLinkPos
 *
 * \param[in] joint		joint name
 * \param[out] getLinkPos	configuration
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *				S_hrp2_BAD_CONFIGURATION
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2GetLinkPosMain(ghrp2_joint_index_t *joint, GHrp2LinkPos *getLinkPos, int *report)
{
#if 0
  int i, j;
  T3D t3d;
  CORBA::Boolean s;
  Configuration c;

  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  /* set model configuration to match actual state of the robot */
  openHRPdyn->setModelJoints(robotState->angle);

  /* get link configuration */
  s = openHRPdyn->getLinkAbsConfiguration(jointNames[*joint].linkname, c);
  if (!s) {
    *report = S_hrp2_NOT_INITIALIZED;
  }

  /* compute euler values */
  t3d.matrix = t3dMatrixIdentity;
  for(i=0;i<3;i++) {
    for(j=0;j<3;j++) t3d.matrix.matrix[j +i*4] = c.rot[j +i*3];
    t3d.matrix.matrix[3 +i*4] = c.trs[0 +i];
  }
  t3dConvertTo(T3D_EULER, &t3d);

  for(i=0;i<3;i++) {
    getLinkPos->t[i] = t3d.euler.euler[3+i];
    getLinkPos->r[i] = t3d.euler.euler[i];
  }
#endif

  return ETHER;
}


/* --- SetLinkPos ---------------------------------------------------- */

/** \brief codel START of SetLinkPos
 *
 * \param[in] setLinkPos	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetLinkPosStart(GHrp2SetLinkPos *setLinkPos, int *report)
{
#if 0
  int i,j;
  T3D t3d;
  CORBA::Boolean s;
  dsequence_var ref;
  Configuration c;

  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  /* convert euler to matrix */
  t3d.euler = t3dEulerIdentity;
  for(i=0;i<3;i++) {
    t3d.euler.euler[i] = setLinkPos->pos.r[i];
    t3d.euler.euler[3+i] = setLinkPos->pos.t[i];
  }
  t3dConvertTo(T3D_MATRIX, &t3d);

  for(i=0;i<3;i++) {
    for(j=0;j<3;j++) c.rot[j +i*3] = t3d.matrix.matrix[j +i*4];
    c.trs[0 +i] = t3d.matrix.matrix[3 +i*4];
  }

  /* compute configuration */
  s = openHRPdyn->moveModelAbs(jointNames[setLinkPos->start].linkname,
			       jointNames[setLinkPos->end].linkname,
			       c);
  if (!s) {
    *report = S_hrp2_BAD_CONFIGURATION;
    return ETHER;
  }

  /* get configuration */
  openHRPdyn->getModelJoints(ref);

  /* send configuration to sequence player plugin */
  openHRPseq->setJointAngles(ref, setLinkPos->delay);
#endif

  return EXEC;
}


/** \brief codel EXEC of SetLinkPos
 *
 * \param[in] setLinkPos	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetLinkPosMain(GHrp2SetLinkPos *setLinkPos, int *report)
{
  if (openHRPseq->isEmpty()) return ETHER;
  return EXEC;
}


/* --- GoHalfSitting ------------------------------------------------- */

/** \brief codel EXEC of GoHalfSitting
 *
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC
 */

ACTIVITY_EVENT
hrp2GoHalfSittingMain(int *report)
{
  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  openHRPseq->goHalfSitting(0);
  return ETHER;
}


/* --- Track --------------------------------------------------------- */

/** hrp2TrackStart  -  codel START of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

static netshring_id_t track_source;

ACTIVITY_EVENT
hrp2TrackStart(GHrp2Track *track, int *report)
{
  track_source = netshring_connect(track->reference, SHRNG_SYNC);
  if (!track_source) {
    fprintf(stderr, "cannot find feed `%s'\n", track->reference);
    *report = S_hrp2_NO_FEED;
    return ETHER;
  }

  fprintf(stdout, "tracking from feed `%s'\n", track->reference);
  return EXEC;
}


/** hrp2TrackMain  -  codel EXEC of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *              S_hrp2_FEED_UNREADABLE
 *              S_hrp2_CANNOT_PUSH
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackMain(GHrp2Track *track, int *report)
{
  unsigned long prev;
  int i, n;
  ghrp2_config_t target;

  /* flood fill the buffer */
  prev = configs.length();
  if (configs.length() > 20) return EXEC;

  do {
    n = netshring_pop(track_source, &target, sizeof(target), SHRNG_ASYNC);
    if (n == 0) break;
    if (n < 0 || n != sizeof(target)) {
      *report = S_hrp2_FEED_UNREADABLE; return END;
    }

    if (target.time == -1.0) {
      fprintf(stdout, "end of motion from feed `%s'\n", track->reference);
      return END;
    }

    n = configs.length();
    configs.length(n+1);
    configs[n].angles.length(GHRP2_MAXDOF);
    configs[n].zmp.length(3);
    configs[n].waistRpy.length(3);

    configs[n].time = target.time;
    configs[n].stable = 0;
    for(i = 0; i<GHRP2_MAXDOF; i++) configs[n].angles[i] = target.angles[i];
    for(i = 0; i<3; i++) configs[n].zmp[i] = target.zmp[i];
    for(i = 0; i<3; i++) configs[n].waistRpy[i] = target.waistRpy[i];

    if (configs.length() > 20) break;
  } while(1);

  if (configs.length() != prev)
    printf("read %lu samples\n", configs.length() - prev);
  return EXEC;
}


/** hrp2TrackEnd  -  codel END of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackEnd(GHrp2Track *track, int *report)
{
  /* disconnect source */
  if (track_source) netshring_disconnect(track_source);
  track_source = NULL;

  return ETHER;
}


/** hrp2TrackInter  -  codel INTER of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackInter(GHrp2Track *track, int *report)
{
  return hrp2TrackEnd(track, report);
}


/* --- TrackDebug ---------------------------------------------------- */

/** hrp2TrackDebugMain  -  codel EXEC of TrackDebug
 *
 * Reports:      OK
 *
 * \return ETHER
 */

ACTIVITY_EVENT
hrp2TrackDebugMain(int *report)
{
  genom_plugin->debug();
  return ETHER;
}
