/*	$Id: hrp2ServoCodels.cc 2007/07/03 14:29:21 mallet $ */

/*
 * Copyright (c) 2007 Anthony Mallet                  --  Fri Jun 22 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

/* --- Codels called by execution task hrp2Servo --------------------- */

#include <err.h>
#include <math.h>
#include <portLib.h>

#include "server/hrp2Header.h"
#include "genom-openhrp.h"


/* --- local data ---------------------------------------------------- */



/* ---hrp2ServoInit -------------------------------------------------- */

/** Initialization codel (fIDS, ...)
 *
 * \returns OK or ERROR
 */

STATUS
hrp2ServoInit(int *report)
{
  return OK;
}


/* --- hrp2ServoPerm ------------------------------------------------- */

/** codel of permanent activity
 *
 * \returns OK or ERROR
 */

STATUS
hrp2ServoPerm(int *report)
{
  return OK;
}


/* --- hrp2ServoEnd -------------------------------------------------- */

/**  Termination codel
 *
 * \returns OK or ERROR
 */

STATUS
hrp2ServoEnd(void)
{
  return OK;
}


/* --- Track --------------------------------------------------------- */

/** hrp2TrackStart  -  codel START of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

static netshring_id_t track_source;

ACTIVITY_EVENT
hrp2TrackStart(GHrp2Track *track, int *report)
{
  track_source = netshring_connect(track->reference);
  if (!track_source) {
    fprintf(stderr, "cannot find feed `%s'\n", track->reference);
    *report = S_hrp2_NO_FEED;
    return ETHER;
  }

  fprintf(stdout, "tracking from feed `%s'\n", track->reference);
  return EXEC;
}


/** hrp2TrackMain  -  codel EXEC of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *              S_hrp2_FEED_UNREADABLE
 *              S_hrp2_CANNOT_PUSH
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackMain(GHrp2Track *track, int *report)
{
  int n;

  /* flood fill the buffer */
  do {
    n = netshring_pop(track_source, &SDI_F->target, sizeof(SDI_F->target));
    if (n == 0) break;
    if (n < 0 || n != sizeof(SDI_F->target)) {
      *report = S_hrp2_FEED_UNREADABLE; return END;
    }

    if (SDI_F->target.time == GHRP2_INVAL_TIME) {
      fprintf(stdout, "end of motion from feed `%s'\n", track->reference);
      return END;
    }

    n = netshring_push(shtarget, &SDI_F->target, sizeof(SDI_F->target));
    if (n == 0) break;
    if (n < 0 || n != sizeof(SDI_F->target)) {
      *report = S_hrp2_CANNOT_PUSH; return END;
    }

    SDI_F->target.time = GHRP2_INVAL_TIME;
  } while(1);

  return EXEC;
}


/** hrp2TrackEnd  -  codel END of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackEnd(GHrp2Track *track, int *report)
{
  int n;

  /* disconnect source */
  if (track_source) netshring_disconnect(track_source);
  track_source = NULL;

  /* push end of sequence marker */
  SDI_F->target.time = GHRP2_INVAL_TIME;
  n = netshring_push(shtarget, &SDI_F->target, sizeof(SDI_F->target));
  if (n == 0) return END;
  if (n < 0 || n != sizeof(SDI_F->target)) {
    *report = S_hrp2_CANNOT_PUSH; return ETHER;
  }
  return ETHER;
}


/** hrp2TrackInter  -  codel INTER of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackInter(GHrp2Track *track, int *report)
{
  return hrp2TrackEnd(track, report);
}
