/*	$Id: joints.h 2007/06/22 12:16:41 mallet $ */

/*
 * Copyright (c) 2006 Anthony Mallet                  --  Fri Jul 28 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_GHRP2_JOINTS
#define H_GHRP2_JOINTS

#include "hrp2Struct.h"

#include "bodyinfo.h"
#include "joints.h"

namespace genom {
  namespace hrp2 {
    /** Mapping between openHRP joint names and genom names */
    static const struct {
      unsigned int genom;
      unsigned int openHRP;
      const char *jointname;
      const char *linkname;
    } jointNames[] = {
      { GHRP2_RLEG_JOINT0,  RLEG_JOINT0, "RLEG_JOINT0", "HRP2JRL_RLEG_JOINT0" },
      { GHRP2_RLEG_JOINT1,  RLEG_JOINT1, "RLEG_JOINT1", "HRP2JRL_RLEG_JOINT1" },
      { GHRP2_RLEG_JOINT2,  RLEG_JOINT2, "RLEG_JOINT2", "HRP2JRL_RLEG_JOINT2" },
      { GHRP2_RLEG_JOINT3,  RLEG_JOINT3, "RLEG_JOINT3", "HRP2JRL_RLEG_JOINT3" },
      { GHRP2_RLEG_JOINT4,  RLEG_JOINT4, "RLEG_JOINT4", "HRP2JRL_RLEG_JOINT4" },
      { GHRP2_RLEG_JOINT5,  RLEG_JOINT5, "RLEG_JOINT5", "HRP2JRL_RLEG_JOINT5" },

      { GHRP2_LLEG_JOINT0,  LLEG_JOINT0, "LLEG_JOINT0", "HRP2JRL_LLEG_JOINT0" },
      { GHRP2_LLEG_JOINT1,  LLEG_JOINT1, "LLEG_JOINT1", "HRP2JRL_LLEG_JOINT1" },
      { GHRP2_LLEG_JOINT2,  LLEG_JOINT2, "LLEG_JOINT2", "HRP2JRL_LLEG_JOINT2" },
      { GHRP2_LLEG_JOINT3,  LLEG_JOINT3, "LLEG_JOINT3", "HRP2JRL_LLEG_JOINT3" },
      { GHRP2_LLEG_JOINT4,  LLEG_JOINT4, "LLEG_JOINT4", "HRP2JRL_LLEG_JOINT4" },
      { GHRP2_LLEG_JOINT5,  LLEG_JOINT5, "LLEG_JOINT5", "HRP2JRL_LLEG_JOINT5" },

      { GHRP2_CHEST_JOINT0, CHEST_JOINT0, "CHEST_JOINT0", "HRP2JRL_CHEST_JOINT0" },
      { GHRP2_CHEST_JOINT1, CHEST_JOINT1, "CHEST_JOINT1", "HRP2JRL_CHEST_JOINT1" },

      { GHRP2_HEAD_JOINT0,  HEAD_JOINT0, "HEAD_JOINT0", "HRP2JRL_HEAD_JOINT0" },
      { GHRP2_HEAD_JOINT1,  HEAD_JOINT1, "HEAD_JOINT1", "HRP2JRL_HEAD_JOINT1" },

      { GHRP2_RARM_JOINT0,  RARM_JOINT0, "RARM_JOINT0", "HRP2JRL_RARM_JOINT0" },
      { GHRP2_RARM_JOINT1,  RARM_JOINT1, "RARM_JOINT1", "HRP2JRL_RARM_JOINT1" },
      { GHRP2_RARM_JOINT2,  RARM_JOINT2, "RARM_JOINT2", "HRP2JRL_RARM_JOINT2" },
      { GHRP2_RARM_JOINT3,  RARM_JOINT3, "RARM_JOINT3", "HRP2JRL_RARM_JOINT3" },
      { GHRP2_RARM_JOINT4,  RARM_JOINT4, "RARM_JOINT4", "HRP2JRL_RARM_JOINT4" },
      { GHRP2_RARM_JOINT5,  RARM_JOINT5, "RARM_JOINT5", "HRP2JRL_RARM_JOINT5" },
      { GHRP2_RARM_JOINT6,  RARM_JOINT6, "RARM_JOINT6", "HRP2JRL_RARM_JOINT6" },

      { GHRP2_LARM_JOINT0,  LARM_JOINT0, "LARM_JOINT0", "HRP2JRL_LARM_JOINT0" },
      { GHRP2_LARM_JOINT1,  LARM_JOINT1, "LARM_JOINT1", "HRP2JRL_LARM_JOINT1" },
      { GHRP2_LARM_JOINT2,  LARM_JOINT2, "LARM_JOINT2", "HRP2JRL_LARM_JOINT2" },
      { GHRP2_LARM_JOINT3,  LARM_JOINT3, "LARM_JOINT3", "HRP2JRL_LARM_JOINT3" },
      { GHRP2_LARM_JOINT4,  LARM_JOINT4, "LARM_JOINT4", "HRP2JRL_LARM_JOINT4" },
      { GHRP2_LARM_JOINT5,  LARM_JOINT5, "LARM_JOINT5", "HRP2JRL_LARM_JOINT5" },
      { GHRP2_LARM_JOINT6,  LARM_JOINT6, "LARM_JOINT6", "HRP2JRL_LARM_JOINT6" },

      { GHRP2_RHAND_JOINT0, RHAND_JOINT0, "RHAND_JOINT0", "HRP2JRL_RHAND_JOINT0" },
      { GHRP2_RHAND_JOINT1, RHAND_JOINT1, "RHAND_JOINT1", "HRP2JRL_RHAND_JOINT1" },
      { GHRP2_RHAND_JOINT2, RHAND_JOINT2, "RHAND_JOINT2", "HRP2JRL_RHAND_JOINT2" },
      { GHRP2_RHAND_JOINT3, RHAND_JOINT3, "RHAND_JOINT3", "HRP2JRL_RHAND_JOINT3" },
      { GHRP2_RHAND_JOINT4, RHAND_JOINT4, "RHAND_JOINT4", "HRP2JRL_RHAND_JOINT4" },

      { GHRP2_LHAND_JOINT0, LHAND_JOINT0, "LHAND_JOINT0", "HRP2JRL_LHAND_JOINT0" },
      { GHRP2_LHAND_JOINT1, LHAND_JOINT1, "LHAND_JOINT1", "HRP2JRL_LHAND_JOINT1" },
      { GHRP2_LHAND_JOINT2, LHAND_JOINT2, "LHAND_JOINT2", "HRP2JRL_LHAND_JOINT2" },
      { GHRP2_LHAND_JOINT3, LHAND_JOINT3, "LHAND_JOINT3", "HRP2JRL_LHAND_JOINT3" },
      { GHRP2_LHAND_JOINT4, LHAND_JOINT4, "LHAND_JOINT4", "HRP2JRL_LHAND_JOINT4" }
    };

    /** Mapping between pom link values and names */
    static const struct { unsigned int value; const char *name; } pomLinkNames[] = {
      { HRP2_HEAD_TO_NECK,	"Head-Neck" },
      { HRP2_CHEST_TO_WAIST,	"Chest-Waist" }
    };
  }
}

#endif /* H_GHRP2_JOINTS */
