/*	$Id: hrp2Struct.h 2007/07/03 18:20:05 mallet $ */

/*
 * Copyright (c) 2006 LAAS/CNRS                       --  Tue Jul 25 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_GENOM_HRP2STRUCT
#define H_GENOM_HRP2STRUCT

#ifdef __cplusplus
extern "C" {
#endif

#include "genom-openhrp/genom-hrp2.h"


/** Link configuration */
typedef struct GHrp2LinkPos {
  double t[3];			/**< translations */
  double r[3];			/**< rotations */
} GHrp2LinkPos;


/* --- Pom sensor estimators ----------------------------------------- */

typedef enum GHrp2PomLinkName {
  HRP2_HEAD_TO_NECK,
  HRP2_CHEST_TO_WAIST,
  HRP2_MAXLINK
} GHrp2PomLinkName;


/* --- Requests parameters ------------------------------------------- */

/** Servo requests parameters */
typedef enum GHrp2ServoStatus {
  HRP2_SERVO_OFF,
  HRP2_SERVO_ON
} GHrp2ServoStatus;

/** SetJointAngle requests parameters */
typedef struct GHrp2SetJointAngle {
  ghrp2_joint_index_t joint;	/**< joint name */
  double angle;			/**< configuration, in radian */
  double delay;			/**< time to go to the specified
				 * configuration */
} GHrp2SetJointAngle;


/** SetLinkPos joint configuration */
typedef struct GHrp2SetLinkPos {
  ghrp2_joint_index_t start;	/**< start joint name */
  ghrp2_joint_index_t end;	/**< moving joint name */
  GHrp2LinkPos pos;		/**< link configuration */
  double delay;			/**< time to go to the specified
				 * configuration */
} GHrp2SetLinkPos;


/** Track request parameters */
typedef struct GHrp2Track {
  char reference[64];		/**< feed name */
} GHrp2Track;


/* --- Magic stuff --------------------------------------------------- */

#define GHRP2_TARGET_CONFIG_NAME	"hrp2-target"
#define GHRP2_TARGET_CONFIG_BUFFER	(1.0 /*seconds of motion*/)

#ifdef __cplusplus
}
#endif

#endif /* H_GENOM_HRP2STRUCT */
